import os
import sys
import pathspec

def load_ignore_patterns(directory_path, filenames=['.gitignore', '.snapshotignore']):
    patterns = []
    
    for filename in filenames:
        ignore_path = os.path.join(directory_path, filename)
        if os.path.exists(ignore_path):
            with open(ignore_path, 'r') as f:
                patterns.extend(f.read().splitlines())
    
    # Always ignore .git directory
    patterns.append(".git")
    
    if not patterns:
        return None
        
    return pathspec.PathSpec.from_lines(pathspec.patterns.GitWildMatchPattern, patterns)

def get_files_content_recursively(directory_path, pathspec=None):
    result = []

    for root, dirs, files in os.walk(directory_path):
        # Ignore directories matching ignore patterns
        if pathspec:
            dirs[:] = [d for d in dirs if not pathspec.match_file(os.path.relpath(os.path.join(root, d), directory_path))]
        
        for file in files:
            relative_path = os.path.relpath(os.path.join(root, file), directory_path)

            # Skip files matching ignore patterns
            if pathspec and pathspec.match_file(relative_path):
                continue

            # Skip hidden files # TODO: Make this configurable
            if file.startswith("."):
                continue

            try:
                with open(os.path.join(root, file), 'r', encoding='utf-8') as f:
                    content = f.read()
            except UnicodeDecodeError:
                content = "Binary file"
            except Exception as e:
                content = f"Error reading file: {e}"
            
            # Indent each line with 4 spaces for Markdown code blocks
            indented_content = "\n".join(f"    {line}" for line in content.splitlines())
            result.append(f"### {relative_path}\n{indented_content}\n")

    return "\n".join(result)

def main():
    if len(sys.argv) != 2:
        print("Usage: python content_snapshot.py <directory_path>")
        sys.exit(1)

    directory_path = sys.argv[1]
    if not os.path.isdir(directory_path):
        print(f"The path {directory_path} is not a valid directory.")
        sys.exit(1)

    ignore_spec = load_ignore_patterns(directory_path)
    result = get_files_content_recursively(directory_path, ignore_spec)
    print(result)

if __name__ == "__main__":
    main()